from django.contrib import admin
from .models import SwiftFile, SwiftContainer
from .forms import SwiftFileForm

# Register your models here.
class SwiftContainerAdmin(admin.ModelAdmin):

	def save_model(self, request, obj, form, change):
		SwiftContainer.create_container(title=obj.title)

class SwiftFileAdmin(admin.ModelAdmin):
	exclude=['date_created', 'filename', 'author', 'filesize', 'content_type', 'container', 'date_modified', 'temp_url', 'key']
	form = SwiftFileForm
	actions = ['delete_selected']

	def save_model(self, request, obj, form, change):
		data = form.cleaned_data['filefield']
		filename=data.name
		content_type = data.content_type
		obj = SwiftFile.upload_file(file_contents=data.read(), filename=filename, content_type=content_type)
		return obj

	def delete_selected(modeladmin, request, queryset):
	    for obj in queryset.all():
	        obj.delete()
		
admin.site.register(SwiftContainer, SwiftContainerAdmin)
admin.site.register(SwiftFile, SwiftFileAdmin)



import time
import urlparse
import hmac
import string
import random
from hashlib import sha1

from swiftclient import client

def get_temp_key(connection):
    """ Tries to get meta-temp-url key from account.
    If not set, generate tempurl and save it to acocunt.
    This requires at least account owner rights. """

    try:
        account = connection.get_account()
    except client.ClientException:
        return None

    key = account[0].get('x-account-meta-temp-url-key')

    if not key:
        chars = string.ascii_lowercase + string.digits
        key = ''.join(random.choice(chars) for x in range(32))
        headers = {'x-account-meta-temp-url-key': key}
        try:
            client.post_account(storage_url, auth_token, headers)
        except client.ClientException:
            return None
    return key


def get_temp_url(connection, filename, container, expires=600):
    key = get_temp_key(connection)
    if not key:
        return None

    expires += int(time.time())
    url_parts = urlparse.urlparse(connection.url)
    path = "%s/%s/%s" % (url_parts.path, container, filename)
    base = "%s://%s" % (url_parts.scheme, url_parts.netloc)
    hmac_body = 'GET\n%s\n%s' % (expires, path)
    sig = hmac.new(key, hmac_body.encode("utf-8"), sha1).hexdigest()
    url = '%s%s?temp_url_sig=%s&temp_url_expires=%s' % (
        base, path, sig, expires)
    return url



from django.test import TestCase
from tastypie.test import ResourceTestCase
from django.db import models
from django.contrib.auth.models import User
import swiftclient
from datetime import datetime
from django.conf import settings
from .models import SwiftContainer, SwiftFile, DEFAULT_CONTAINER_TITLE
import os

USERNAME = settings.SW_USERNAME
KEY = settings.SW_KEY
AUTH_URL = settings.SW_AUTH_URL

class SwiftContainerTestCase(TestCase):
	def setUp(self):
		super(self.__class__, self).setUp()

	def test_create_default_container(self):
		container = SwiftContainer.create_default_container()
		self.assertEqual(container.title, DEFAULT_CONTAINER_TITLE)
		self.assertEqual(container.service_slug, USERNAME)
		try:
			conn = swiftclient.Connection(user=USERNAME, key=KEY, authurl=AUTH_URL)
			_m, objects = conn.get_container(container.title)
			self.assertEqual(len(objects), 0)
		except swiftclient.ClientException:
			raise Exception("Access denied")
		else:
			container.delete()
			self.assertEqual(SwiftContainer.objects.all().count(), 0)

class SwiftFileTestCase(TestCase):
	def setUp(self):
		super(self.__class__, self).setUp()

	def test_upload(self):
		file_path = os.path.join(os.path.dirname(os.path.dirname(__file__)),
								 'almastorage/__init__.py')
		from mimetypes import MimeTypes
		mime = MimeTypes()
		mime_type = mime.guess_type(file_path)
		f = open(file_path, "rb")

		swf = SwiftFile.upload_file(file_contents=f.read(), filename='__init__.py', content_type=mime_type)

		self.assertEqual(SwiftContainer.objects.all().count(), 1)
		self.assertEqual(SwiftContainer.objects.first().title, DEFAULT_CONTAINER_TITLE)

		try:
			conn = swiftclient.Connection(user=USERNAME, key=KEY, authurl=AUTH_URL)
			_m, objects = conn.get_container(DEFAULT_CONTAINER_TITLE)
			self.assertEqual(len(objects), 1)
		except swiftclient.ClientException:
			raise Exception("Access denied")
		else:
			SwiftContainer.objects.first().delete()
			self.assertEqual(SwiftContainer.objects.all().count(), 0)
			self.assertEqual(SwiftFile.objects.all().count(), 0)

	def test_temp_url(self):
		file_path = os.path.join(os.path.dirname(os.path.dirname(__file__)),
								 'almastorage/__init__.py')
		from mimetypes import MimeTypes
		mime = MimeTypes()
		mime_type = mime.guess_type(file_path)
		f = open(file_path, "rb")

		swf = SwiftFile.upload_file(file_contents=f.read(), filename='__init__.py', content_type=mime_type)

		url = swf.get_temp_download_url()
		swf.delete()

		try:
			conn = swiftclient.Connection(user=USERNAME, key=KEY, authurl=AUTH_URL)
			_m, objects = conn.get_container(DEFAULT_CONTAINER_TITLE)
			self.assertEqual(len(objects), 0)
		except swiftclient.ClientException:
			raise Exception("Access denied")
		else:
			SwiftContainer.objects.first().delete()
			self.assertEqual(SwiftContainer.objects.all().count(), 0)

class AlmastorageResourceTest(ResourceTestCase):

	def setUp(self):
		super(self.__class__, self).setUp()

		self.api_path_container= '/api/v1/storage/'

		self.api_path_files = '/api/v1/files/'

		self.get_resp_files = lambda path: self.api_client.get(
			self.api_path_files + path,
			format='json',
			HTTP_HOST='localhost')
		self.get_des_res_files = lambda path: self.deserialize(self.get_resp_files(path))


		self.get_resp = lambda path: self.api_client.get(
			self.api_path_container + path,
			format='json',
			HTTP_HOST='localhost')
		self.get_des_res = lambda path: self.deserialize(self.get_resp(path))

	def test_upload_and_download(self):
		container = SwiftContainer.create_default_container()
		self.assertEqual(container.title, DEFAULT_CONTAINER_TITLE)
		self.assertEqual(container.service_slug, USERNAME)
		try:
			conn = swiftclient.Connection(user=USERNAME, key=KEY, authurl=AUTH_URL)
			_m, objects = conn.get_container(container.title)
			# self.assertEqual(len(objects), 0)
		except swiftclient.ClientException:
			raise Exception("Access denied")
		else:
			file_path = os.path.join(os.path.dirname(os.path.dirname(__file__)),
								 'almastorage/models.py')
			from mimetypes import MimeTypes
			mime = MimeTypes()
			mime_type = mime.guess_type(file_path)
			f = open(file_path, "rb")
			filename='models.py'

			import base64
			post_data={
				'filename': filename,
				'content_type': mime_type,
				'uploaded_file': base64.b64encode(open(file_path, "rb").read())
				}

			resp = self.api_client.post(
				self.api_path_container+"%s/upload/"%container.service_slug, format='json', data=post_data)
			des_resp = self.deserialize(resp)

			conn = swiftclient.Connection(user=USERNAME, key=KEY, authurl=AUTH_URL)
			_m, objects = conn.get_container(DEFAULT_CONTAINER_TITLE)
			# self.assertEqual(len(objects), 1)
			resp = self.api_client.get(self.api_path_files+str(des_resp['file']['id'])+'/download/',
										format='json')
			des_resp = self.deserialize(resp)

			import requests
			r = requests.get(des_resp['temp_url'], stream=True)
			context = ""
			for chunk in r.iter_content():
				context += chunk
			self.assertTrue(len(context) != 0)
			container.delete()
			self.assertEqual(SwiftContainer.objects.all().count(), 0)


from django.shortcuts import render, render_to_response
from django.views.generic import ListView
from .models import SwiftFile, SwiftContainer
from .forms import UploadFileForm
from django.http import HttpResponseRedirect
from .models import *
import swiftclient
from django.contrib.auth.models import User
from django.http import HttpResponse
from django.core.servers.basehttp import FileWrapper

class ContainerListView(ListView):
    model = SwiftContainer
    paginate_by = 10
    queryset = SwiftFile.objects.all()
    template_name='swift_file/templates/swift_file/swiftfile_list.html'
    context_object_name = 'container'

def upload_file(request):
    if request.method == 'GET':
        return render(request, 'swift_file/upload_form.html',{'form':UploadFileForm()})
    elif request.method == 'POST':
        form = UploadFileForm(request.POST, request.FILES)
        conn = swiftclient.Connection(user=USER, key=KEY, authurl=AUTHURL)
        SwiftFile.upload_file(User.objects.first(), conn, request.FILES['file'].read(), request.FILES['file'].name, request.FILES['file'].content_type)
        if form.is_valid():
            return HttpResponseRedirect('/')
        pass

def download_file(request, file_id):
    if request.method == 'GET':
        base_file = SwiftFile.objects.get(id=file_id)
        filename = base_file.filename.split('.')[0]+'_'+base_file.date.time().__str__()+'.'+base_file.filename.split('.')[1]
        conn = swiftclient.Connection(user=USER, key=KEY, authurl=AUTHURL)
        obj_tuple = conn.get_object('images', filename)
        response = HttpResponse(obj_tuple[1], content_type=base_file.content_type)
        response['Content-Disposition'] = 'attachment; filename='+base_file.filename
        return response




from tastypie import fields, http
from tastypie.resources import Resource, ModelResource, ALL, ALL_WITH_RELATIONS
from tastypie.serializers import Serializer
from .models import SwiftContainer, SwiftFile
from django.conf.urls import url
from tastypie.utils import trailing_slash
import datetime
import base64
import simplejson as json
import swiftclient

class SwiftContainerResource(ModelResource):

	class Meta:
		queryset = SwiftContainer.objects.all()
		resource_name = 'storage'
		detail_allowed_methods = ['get', 'post']

	def prepend_urls(self):
		return [
			url(
				r"^(?P<resource_name>%s)/(?P<slug>\w+)/upload%s$" %
				(self._meta.resource_name, trailing_slash()),
				self.wrap_view('upload'),
				name='api_upload'
			),
		]

	def upload(self, request, **kwargs):
		data = self.deserialize(
			request, request.body,
			format=request.META.get('CONTENT_TYPE', 'application/json'))
		file_contents = base64.b64decode(data['uploaded_file'])
		filename = data['filename']
		content_type = data['content_type']
		author = request.user if not request.user.is_anonymous() else None

		try:
			swiftfile = SwiftFile.upload_file(file_contents=file_contents, filename=filename, 
											content_type=content_type, author=author)
		except swiftclient.ClientException:
			return http.HttpUnauthorized("You are not authorized in stackswift service, \
				please, make sure that you add your username and key to your settings")

		bundle = SwiftFileResource().build_bundle(obj=swiftfile, request=request)
		bundle = SwiftFileResource().full_dehydrate(bundle)
		
		return self.create_response(
			request, {
				'file': bundle
			},
			response_class=http.HttpAccepted)


class SwiftFileResource(ModelResource):

	class Meta:
		queryset = SwiftFile.objects.all()
		resource_name = 'files'
		detail_allowed_methods = ['get', 'post']

	def prepend_urls(self):
		return [
			url(
				r"^(?P<resource_name>%s)/(?P<id>\d+)/download%s$" %
				(self._meta.resource_name, trailing_slash()),
				self.wrap_view('download'),
				name='api_download'
			),
		]

	def download(self, request, **kwargs):
		swiftfile = SwiftFile.objects.get(pk=kwargs.get('id'))
		try:
			temp_url =swiftfile.get_temp_download_url()
		except swiftclient.ClientException:
			return http.HttpUnauthorized("You are not authorized in stackswift service, \
				please, make sure that you add your username and key to your settings")
		return self.create_response(
			request, {
				'temp_url': temp_url
			},
			response_class=http.HttpAccepted)


import os
from setuptools import setup

README = open(os.path.join(os.path.dirname(__file__), 'README.md')).read()

# allow setup.py to be run from any path
os.chdir(os.path.normpath(os.path.join(os.path.abspath(__file__), os.pardir)))

setup(
    name='django-almastorage',
    version='0.1.1',
    packages=['almastorage'],
    include_package_data=True,
    description='A simple Django app to use SwiftStack Storage',
    long_description=README,
    url='https://github.com/aaannnyyy/django-almastorage',
    author='Nurlan Abyken',
    author_email='abyken.nurlan@gmail.com',
    install_requires=['django>=1.5', 'python-swiftclient', 'django-tastypie', 'requests'],
    zip_safe=False,
    classifiers=[
        'Environment :: Web Environment',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        'Topic :: Utilities',
    ],
)


from django.db import models
from django.contrib.auth.models import User
import swiftclient
from datetime import datetime
from django.conf import settings
import utils

USERNAME = settings.SW_USERNAME
KEY = settings.SW_KEY
AUTH_URL = settings.SW_AUTH_URL

USER_MODEL = settings.AUTH_USER_MODEL
DEFAULT_CONTAINER_TITLE = 'Main_container'

class SwiftContainer(models.Model):
	title = models.CharField(max_length=255, default = DEFAULT_CONTAINER_TITLE)
	service_slug = models.CharField('service slug', max_length=30, unique=True, blank=False, default = USERNAME)
	date_created = models.DateTimeField(auto_now_add=True)

	class Meta:
		db_table = 'sw_container'
		ordering = ['-date_created']
		verbose_name = 'sw_container'
		verbose_name_plural = 'sw_containers'

	def __unicode__(self):
		return self.title

	@classmethod 
	def create_default_container(cls):
		try:
			container = cls(title=DEFAULT_CONTAINER_TITLE, service_slug=USERNAME)
			conn = swiftclient.Connection(user=USERNAME, key=KEY, authurl=AUTH_URL)
			conn.put_container(container.title)
			container.save()
		except swiftclient.ClientException:
			raise Exception("Access denied")

		return container

	@classmethod 
	def create_container(cls, title):
		try:
			container = cls(title=title, service_slug=USERNAME)
			conn = swiftclient.Connection(user=USERNAME, key=KEY, authurl=AUTH_URL)
			conn.put_container(container.title)
			container.save()
		except swiftclient.ClientException:
			raise Exception("Access denied")

		return container

	def delete(self, **kwargs):
		try:
			conn = swiftclient.Connection(user=USERNAME, key=KEY, authurl=AUTH_URL)
			_m, objects = conn.get_container(self.title)
			for obj in objects:
				conn.delete_object(self.title, obj['name'])
			conn.delete_container(self.title)
		except swiftclient.ClientException:
			raise Exception("Access denied")
		super(self.__class__, self).delete(**kwargs)


class SwiftFile(models.Model):
	date_created = models.DateTimeField(auto_now_add=True)
	filename = models.CharField(max_length=255)
	author = models.ForeignKey(USER_MODEL, verbose_name='user', related_name='files_set', blank=True, null=True)
	filesize = models.IntegerField(blank=True, null=True)
	content_type = models.CharField(max_length=100)
	container = models.ForeignKey('SwiftContainer', related_name='files')
	date_modified = models.DateTimeField(auto_now=True, default=datetime.now())
	temp_url = models.CharField(max_length=255)
	key = models.CharField(max_length=40)

	class Meta:
		db_table = 'sw_file'
		ordering = ['-date_created']
		verbose_name = 'sw_file'
		verbose_name_plural = 'sw_files'

	def __unicode__(self):
		return self.filename 

	@classmethod
	def upload_file(cls, file_contents, filename, content_type,  author=None):
		f = cls(author=author)
		f.filename=filename
		f.content_type=content_type
		f.key = f.generate_key()
		try:
			container = SwiftContainer.objects.get(title=DEFAULT_CONTAINER_TITLE, service_slug=USERNAME)
		except SwiftContainer.DoesNotExist:
			container = SwiftContainer.create_default_container()
		try:
			conn = swiftclient.Connection(user=USERNAME, key=KEY, authurl=AUTH_URL)
			conn.put_object(container.title, f.key, contents=file_contents, content_type=content_type)
		except swiftclient.ClientException:
			raise Exception("Access denied")
		f.container = container
		f.save()
		return f

	@classmethod
	def search_files(cls, filename):
		return cls.objects.filter(filename__startswith=filename)


	def get_temp_download_url(self):
		try:
			conn = swiftclient.Connection(user=USERNAME, 
											key=KEY, authurl=AUTH_URL)
			url = utils.get_temp_url(connection = conn, filename=self.key, container=self.container.title, expires=7 * 24 * 3600)
		except swiftclient.ClientException:
			raise Exception("Access denied")
		return url

	def delete(self, **kwargs):
		try:
			conn = swiftclient.Connection(user=USERNAME, 
											key=KEY, authurl=AUTH_URL)
			conn.delete_object(self.container.title, self.key)
		except swiftclient.ClientException:
			raise Exception("Access denied")
		super(self.__class__, self).delete(**kwargs)

	def generate_key(self):
		import hashlib, random
		salt = hashlib.sha1(str(random.random())).hexdigest()[:5]
		filename = str(self.filename)
		if isinstance(filename, unicode):
			filename = filename.encode('utf-8')
		content_type = str(self.content_type)
		if isinstance(content_type, unicode):
			content_type = content_type.encode('utf-8')
		key = hashlib.sha1(salt+filename+content_type).hexdigest()

		return key

	def save(self, **kwargs):
		self.temp_url = self.get_temp_download_url()
		if not self.key:
			self.key = self.generate_key()
		super(self.__class__, self).save(**kwargs)


